
#include "rfcc.h"

//! Wait CCxxxx ready
#define RFCC_WAIT_READY()	loop_until_bit_is_set(SPI_PORT, SPI_MISO)

namespace rfcc
{
	void init(settings_t *p)
	{
		rfspi::init();

		powerupReset();

		rfspi::writeReg( RFCC_FSCTRL0, p->FSCTRL0 );
		rfspi::writeReg( RFCC_FREQ2, p->FREQ2 );
		rfspi::writeReg( RFCC_FREQ1, p->FREQ1 );
		rfspi::writeReg( RFCC_FREQ0, p->FREQ0 );
		rfspi::writeReg( RFCC_MDMCFG4, p->MDMCFG4 );
		rfspi::writeReg( RFCC_MDMCFG3, p->MDMCFG3 );
		rfspi::writeReg( RFCC_MDMCFG2, p->MDMCFG2 );
		rfspi::writeReg( RFCC_MDMCFG1, p->MDMCFG1 );
		rfspi::writeReg( RFCC_MDMCFG0, p->MDMCFG0 );
		rfspi::writeReg( RFCC_CHANNR, p->CHANNR );
		rfspi::writeReg( RFCC_DEVIATN, p->DEVIATN );
		rfspi::writeReg( RFCC_FREND1, p->FREND1 );
		rfspi::writeReg( RFCC_FREND0, p->FREND0 );
		rfspi::writeReg( RFCC_MCSM2, p->MCSM2 );
		rfspi::writeReg( RFCC_MCSM1, p->MCSM1 );
		rfspi::writeReg( RFCC_MCSM0, p->MCSM0 );
		rfspi::writeReg( RFCC_FOCCFG, p->FOCCFG );
		rfspi::writeReg( RFCC_BSCFG, p->BSCFG );
		rfspi::writeReg( RFCC_AGCCTRL2, p->AGCCTRL2 );
		rfspi::writeReg( RFCC_AGCCTRL1, p->AGCCTRL1 );
		rfspi::writeReg( RFCC_AGCCTRL0, p->AGCCTRL0 );
		rfspi::writeReg( RFCC_FSCAL3, p->FSCAL3 );
		rfspi::writeReg( RFCC_FSCAL2, p->FSCAL2 );
		rfspi::writeReg( RFCC_FSCAL1, p->FSCAL1 );
		rfspi::writeReg( RFCC_FSCAL0, p->FSCAL0 );
		rfspi::writeReg( RFCC_FSTEST, p->FSTEST );
		rfspi::writeReg( RFCC_TEST2, p->TEST2 );
		rfspi::writeReg( RFCC_TEST1, p->TEST1 );
		rfspi::writeReg( RFCC_TEST0, p->TEST0 );
		rfspi::writeReg( RFCC_IOCFG2, p->IOCFG2 );
		rfspi::writeReg( RFCC_IOCFG0, p->IOCFG0 );
		rfspi::writeReg( RFCC_PKTCTRL1, p->PKTCTRL1 );
		rfspi::writeReg( RFCC_PKTCTRL0, p->PKTCTRL0 );
		rfspi::writeReg( RFCC_ADDR, p->ADDR );
		rfspi::writeReg( RFCC_PKTLEN, p->PKTLEN );
		rfspi::writeReg( RFCC_FIFOTHR, p->FIFOTHR );
	}

	void powerupReset()
	{
		SPI_CS_DISABLE();
		_delay_us(40);
		SPI_CS_ENABLE();
		_delay_us(40);
		SPI_CS_DISABLE();
		_delay_us(45);
		SPI_CS_ENABLE();

		RFCC_WAIT_READY();
		spi::transfer(RFCC_SRES);
		RFCC_WAIT_READY();
		SPI_CS_DISABLE();
		_delay_ms(45);
	}

	void sendPacket(char *txBuffer, char size)
	{
		rfspi::writeBurstReg(RFCC_TXFIFO, txBuffer, size);	// Write TX data
		rfspi::strobe(RFCC_STX);							// Change state to TX, initiating
	}

	char receivePacket(char *rxBuffer, char *length)
	{
		char status[2];
		char pktLen;
			
		if ((rfspi::readStatus(RFCC_RXBYTES) & RFCC_NUM_RXBYTES))
		{
			
			pktLen = rfspi::readReg(RFCC_RXFIFO);	// Read length byte
			if (pktLen <= *length)					// If pktLen size <= rxBuffer
			{
				rfspi::readBurstReg(RFCC_RXFIFO, rxBuffer, pktLen);	// Pull data
				*length = pktLen;					// Return the actual size
				rfspi::readBurstReg(RFCC_RXFIFO, status, 2);
				LQI  = status[1];
				RSSI = status[0];					// Read appended status bytes
				return (char)(status[RFCC_LQI_RX]&RFCC_CRC_OK);
			}										// Return CRC_OK bit
			else
			{
				*length = pktLen;					// Return the large size
				rfspi::strobe(RFCC_SFRX);			// Flush RXFIFO
				return 0;							// Error
			}
		}
		else
		{
			return 0;								// Error
		}
	}

	namespace rfspi
	{
		void init()
		{
			spi::init();
		}

		void writeReg(char addr, char value)
		{
			SPI_CS_ENABLE();
			RFCC_WAIT_READY();
			spi::transfer(addr);	// Send address
			spi::transfer(value);	// Send data
			SPI_CS_DISABLE();
		}

		void writeBurstReg(char addr, char *buffer, char count)
		{
			char i;

			SPI_CS_ENABLE();
			RFCC_WAIT_READY();
			spi::transfer(addr | RFCC_WRITE_BURST);	// Send address
			for (i = 0; i < count; i++)
				spi::transfer(buffer[i]);			// Send data
			SPI_CS_DISABLE();
		}

		char readReg(char addr)
		{
			char x;

			SPI_CS_ENABLE();
			RFCC_WAIT_READY();
			spi::transfer(addr | RFCC_READ_SINGLE);	//Send address
			x = spi::transfer();					// Read data
			SPI_CS_DISABLE();
			return x;
		}

		void readBurstReg(char addr, char *buffer, char count)
		{
			char i;

			SPI_CS_ENABLE();
			RFCC_WAIT_READY();
			spi::transfer(addr | RFCC_READ_BURST);	// Send address
			for (i = 0; i < count; i++)
				buffer[i] = spi::transfer();		// Read data
			SPI_CS_DISABLE();
		}

		char readStatus(char addr)
		{
			char x;

			SPI_CS_ENABLE();
			RFCC_WAIT_READY();
			spi::transfer(addr | RFCC_READ_BURST);	// Send address
			x = spi::transfer();					// Read data
			SPI_CS_DISABLE();
			return x;
		}

		void strobe(char strobe_)
		{
			SPI_CS_ENABLE();
			RFCC_WAIT_READY();
			spi::transfer(strobe_);	// Send strobe
			SPI_CS_DISABLE();
		}
	}; // namespace rfspi
};
