// spi.c

#include "spi.h"

/// Loop until any current SPI transmissions have completed
#define spi_wait()	while (!(SPSR & (1 << SPIF)));

/// Initialize the SPI as master
void spi::init()
{
	// make the MOSI, SCK, and SS pins outputs
	SPI_DDR |= ( 1 << SPI_MOSI ) | ( 1 << SPI_SCK ) | ( 1<< SPI_SS );

	// make sure the MISO pin is input
	SPI_DDR &= ~( 1 << SPI_MISO );

	// set up the SPI module: SPI enabled, MSB first, master mode,
	//  clock polarity and phase = 0, F_osc/4
	SPCR = ( 1 << SPE ) | ( 1 << MSTR ) | ( 0 << SPR1 ) | ( 0 << SPR0 );
	
	// set up the SPI module: SPI enabled, MSB first, master mode,
	//  clock polarity and phase = 0, F_osc/16
//	SPCR = ( 1 << SPE ) | ( 1 << MSTR ) | ( 0 << SPR1 ) | ( 1 << SPR0 );

//	SPI_SPSR = 1;     // set double SPI speed for F_osc/2 or F_osc/8
}

/// Transfer a byte of data 
char spi::transfer( char data )
{
	// Start transmission
	SPDR = data;

	// Wait for the transmission to complete
	spi_wait();

	return SPDR;
}

