/*!
 * SPI master routines
 *
 * Add to \b settings.h :
 * @code
 *	/// spi module configuration (ATMega8)
 *	#define SPI_PORT	PORTB
 *	#define SPI_DDR		DDRB
 *	#define SPI_SCK		PB5
 *	#define SPI_MISO	PB4
 *	#define SPI_MOSI	PB3
 *	#define SPI_SS		PB2
 *	@endcode
 */

//!
#ifndef SPI_H
#define SPI_H

#include "settings.h"

//! /CS = 0
#define SPI_CS_ENABLE()		SPI_PORT &= ~(1<<SPI_SS)
//! /CS = 1
#define SPI_CS_DISABLE()	SPI_PORT |=  (1<<SPI_SS)

namespace spi
{
	/// Initialize the SPI subsystem
	void init();

	/// Transfer a byte of data
	char transfer( char data );
	inline char transfer()
	{
		return transfer(0);
	}
}

#endif // SPI_H
